// FGGE Game Example
var invertRotationKey = GameEngine.Keyboard.KeyboardEvent.KEY_SPACE;
var changeRadiusKey = GameEngine.Keyboard.KeyboardEvent.KEY_ENTER;

GameEngine.Renderer.generateCanvas();
GameEngine.Renderer.setCanvasSize(300, 300);
GameEngine.Renderer.setBackgroundColor("black");
GameEngine.Renderer.setFramerate(20);

var ball = new GameEngine.Renderer.Model("ball.png");
var time = 0;
var speed = 0.02;
var rotation = 1;
var numberOfSpheres = 3;
var pseudo3D = false;

function gameFunction() {
	GameEngine.Renderer.clearScreen();
	
	// NOTE: 2PI Radians = 360 Degrees
	for(var i=0; i<(Math.PI*2); i+=(Math.PI*2)/numberOfSpheres) {
		ball.x = Math.sin(time + i)*100;
		ball.y = Math.cos(time + i)*100;
		
		if(pseudo3D) {
			ball.x *= Math.sin(time);
			ball.y *= Math.cos(time);
		}
		
		ball.x += GameEngine.Renderer.canvas.width/2;
		ball.y += GameEngine.Renderer.canvas.height/2;
		
		ball.render();
	}
	
	// NOTE: Pick the higher number between 0 and MousePosition (to remove negative numbers),
	//       then pick the lower between the result and CanvasSize (to remove numbers outside the canvas).
	//       This operation bounds detected mouse position inside the canvas.
	var mouseX = Math.min(Math.max(0, GameEngine.Mouse.x), GameEngine.Renderer.canvas.width)
	var mouseY = Math.min(Math.max(0, GameEngine.Mouse.y), GameEngine.Renderer.canvas.height);
				
	
	// NOTE: floor() removes floating point from numbers.
	numberOfSpheres = Math.floor(mouseX/GameEngine.Renderer.canvas.width * 19) + 1;
	
	speed = mouseY/10000;
	
	time += speed*rotation;
}

function keyboardListener(event) {
	if(event.getEventType() == GameEngine.Keyboard.KeyboardEvent.EVENT_RELEASE) {
		// If space bar is pressed invert rotation,
		// If enter key is pressed switch pseudo 3D mode,
		// Else print pressed key ID.
		if(event.getKeyNumber() == invertRotationKey)
			rotation *= -1;
		else if(event.getKeyNumber() == changeRadiusKey)
			pseudo3D = !pseudo3D;
		else
			alert("Key "+event.getKeyNumber()+" just released!");
	}
}

// Mouse listener not needed: mouse coordinates can always be retrieved.
GameEngine.Keyboard.addKeyboardListener(keyboardListener);
GameEngine.startGame(gameFunction);