// FGGE Game Example
var player1MoveUp = GameEngine.Keyboard.KeyboardEvent.KEY_W;
var player1MoveDown = GameEngine.Keyboard.KeyboardEvent.KEY_S;
var player2MoveUp = GameEngine.Keyboard.KeyboardEvent.KEY_UP;
var player2MoveDown = GameEngine.Keyboard.KeyboardEvent.KEY_DOWN;

GameEngine.Renderer.generateCanvas();
GameEngine.Renderer.setCanvasSize(800, 600);
GameEngine.Renderer.setBackgroundColor("black");
GameEngine.Renderer.setFramerate(25);

var whistleSound = new GameEngine.Media.Audio("whistle.wav");

var c2d = GameEngine.Renderer.context2D;
c2d.fillStyle = "white";
c2d.textBaseline = "top";
c2d.font = "bold 45px sans-serif";

var playerPosition1, playerPosition2;

var playerScore1 = 0, playerScore2 = 0;

var player1Moving, player2Moving;

var ballX, ballY;
var ballSpeedX, ballSpeedY;
var ballSpeedMultiplier;
var playerJustScored = 0;

// Explanation at http://stackoverflow.com/a/402010
function boundsToCircleCollision(posX, posY, width, height, circleX, circleY, radius) {
	var circleDistanceX = Math.abs(circleX - posX);
    var circleDistanceY = Math.abs(circleY - posY);

    if (circleDistanceX > (width/2 + radius) || circleDistanceY > (height/2 + radius))
		return false;

    if (circleDistanceX <= (width/2) || circleDistanceY <= (height/2))
		return true;

    cornerDistance_sq = (circleDistanceX - width/2)^2 + (circleDistanceY - height/2)^2;

    return cornerDistance_sq <= (radius^2);
}

function resetGame() {
	ballX = GameEngine.Renderer.canvas.width/2;
	ballY = GameEngine.Renderer.canvas.height/2;
	
	ballSpeedX = Math.random()*5+5;
	ballSpeedY = Math.random()*10-5;
	
	// If game just started (playerJustScored is 0), ball direction is random.
	if(playerJustScored == 0 && Math.random() > 0.5)
		ballSpeedX *= -1;
		
	// If player 1 just scored ball goes right, else left.
	if(playerJustScored == 2)
		ballSpeedX *= -1;
	
	playerPosition1 = GameEngine.Renderer.canvas.height/2;
	playerPosition2 = GameEngine.Renderer.canvas.height/2;
	
	player1Moving = 0;
	player2Moving = 0;
	
	ballSpeedMultiplier = 1;
}

// NOTE: There is a bug that may cause the ball to get trapped inside a bouncer, this is not fixed just to keep the code simple.
function gameFunction() {
	GameEngine.Renderer.clearScreen();
	
	// Move players and check that they are inside the scene.
	playerPosition1 += player1Moving;
	playerPosition2 += player2Moving;
	
	if(playerPosition1 < 25)
		playerPosition1 = 25;
	else if(playerPosition1 > GameEngine.Renderer.canvas.height-25)
		playerPosition1 = GameEngine.Renderer.canvas.height-25;
		
	if(playerPosition2 < 25)
		playerPosition2 = 25;
	else if(playerPosition2 > GameEngine.Renderer.canvas.height-25)
		playerPosition2 = GameEngine.Renderer.canvas.height-25;
	
	// Check if ball is bouncing on walls.
	// NOTE: 10 is ballRadius/2.
	if(ballY - 10 <= 0 || ballY + 10 >= GameEngine.Renderer.canvas.height)
		ballSpeedY *= -1;
	
	// Check if ball is bouncing on players.
	if(boundsToCircleCollision(20, playerPosition1, 20, 50, ballX, ballY, 10)) {
		ballSpeedX *= -1;
		ballSpeedY = (ballY-playerPosition1)/20;
	}
		
	if(boundsToCircleCollision(GameEngine.Renderer.canvas.width-20, playerPosition2, 20, 50, ballX, ballY, 10)) {
		ballSpeedX *= -1;
		ballSpeedY = (ballY-playerPosition2)/20;
	}
	
	// Check if ball is getting out of scene.
	if(ballX < -10) {
		playerScore2++;
		playerJustScored = 2;
		resetGame();
		
		// Stop to reset sound, play to replay.
		whistleSound.stop();
		whistleSound.play();
	} else if(ballX > GameEngine.Renderer.canvas.width+10) {
		playerScore1++;
		playerJustScored = 1;
		resetGame();
		
		whistleSound.stop();
		whistleSound.play();
	}
	
	// Make ball move
	ballX += ballSpeedX*ballSpeedMultiplier;
	ballY += ballSpeedY*ballSpeedMultiplier;
	
	// Draw player 1
	c2d.beginPath();
	c2d.rect(10, playerPosition1-25, 20, 50);
	c2d.fill();
	
	// Draw player 2
	c2d.beginPath();
	c2d.rect(GameEngine.Renderer.canvas.width-30, playerPosition2-25, 20, 50);
	c2d.fill();
	
	// Draw ball
	c2d.beginPath();
	c2d.arc(ballX, ballY, 10, Math.PI*2, 0, true);
	c2d.closePath();
	c2d.fill();
	
	// Draw Scores
	c2d.textAlign = "right";
	c2d.fillText(playerScore1, GameEngine.Renderer.canvas.width/2-20, 20);

	c2d.textAlign = "left";
	c2d.fillText(playerScore2, GameEngine.Renderer.canvas.width/2+20, 20);
	
	// Make ball go faster and faster so that the game becomes harder.
	ballSpeedMultiplier *= 1.001;
}

function keyboardListener(event) {
	if(event.getEventType() == GameEngine.Keyboard.KeyboardEvent.EVENT_PRESS) {
		if(event.getKeyNumber() == player1MoveUp)
			player1Moving = -10;
		else if(event.getKeyNumber() == player1MoveDown)
			player1Moving = +10;
		else if(event.getKeyNumber() == player2MoveUp)
			player2Moving = -10;
		else if(event.getKeyNumber() == player2MoveDown)
			player2Moving = +10;
	} else if(event.getEventType() == GameEngine.Keyboard.KeyboardEvent.EVENT_RELEASE) {
		if(event.getKeyNumber() == player1MoveUp || event.getKeyNumber() == player1MoveDown)
			player1Moving = 0;
		else if(event.getKeyNumber() == player2MoveUp || event.getKeyNumber() == player2MoveDown)
			player2Moving = 0;
	}
}

GameEngine.Keyboard.addKeyboardListener(keyboardListener);
GameEngine.startGame(gameFunction);

resetGame();