/* FGGE - Fabiogiopla Game Engine
 * Licensed under MIT license.
 * http://fabiogiopla.altervista.org/?cat=software/dev_utils/fgge
 * Copyright (c) Fabio Iotti 2012. Some rights reserved.
 */

var GameEngine = {
	// Static Classes
	Renderer: {
		// Classes
		Model:function(path) {
			// Attributes
			this.x = 0;
			this.y = 0;
			this.transparency = 1;
			this.rotation = 0;
			this.scaleX = 1;
			this.scaleY = 1;
			
			// MODEL IMAGE START
			var imageModel = new Image();
			GameEngine._totalObjects++;
			imageModel.onload = function() {
				GameEngine._loadedObjects++;
				GameEngine._resetGameFunction();
			};
			imageModel.onerror = function() {
				throw { 
					name: "ImageLoadException", 
					message: "Failed to load \""+imageModel.src+"\" image!", 
				}
				
				GameEngine._loadedObjects++;
				GameEngine._resetGameFunction();
			};
			imageModel.src = path;
			// MODEL IMAGE END
			
			// Methods
			this.render = function() {
				var c2d = GameEngine.Renderer.context2D;
				
				c2d.globalAlpha = this.transparency;
				
				if(this.rotation || this.scaleX != 1 || this.scaleY != 1) {
					c2d.save();
					
					c2d.translate(
						this.x,
						this.y
					);
					
					c2d.rotate(this.rotation);
					
					c2d.drawImage(
						imageModel,
						0,
						0,
						imageModel.width,
						imageModel.height,
						(-imageModel.width*this.scaleX)/2,
						(-imageModel.height*this.scaleY)/2,
						imageModel.width*this.scaleX,
						imageModel.height*this.scaleY
					);
					
					c2d.restore();
				} else {
					c2d.drawImage(
						imageModel,
						this.x - imageModel.width/2,
						this.y - imageModel.height/2
					);
				}
			}
		},
		
		// Attributes
		//_hardwareAccelerated
		//canvas
		//context2D
		//contextGL
		
		// Methods
		generateCanvas:function() {
			var newCanvas = document.createElement("canvas");
			document.getElementsByTagName("script")[document.getElementsByTagName("script").length-1].parentNode.appendChild(newCanvas);
			GameEngine.Renderer.setCanvas(newCanvas);
		},
		
		setCanvas:function(canvasElement) {
			GameEngine.Renderer.canvas = canvasElement;
			GameEngine.Renderer.context2D = GameEngine.Renderer.canvas.getContext('2d');
			
			var contextNames = ["webgl", "experimental-webgl", "moz-webgl", "webkit-3d"];
			for(var i=0; i<contextNames.length; i++) {
				try {
					GameEngine.Renderer.contextGL = canvas.getContext(contextNames[i]);
					
					if(GameEngine.Renderer.contextGL)
						break;
				} catch(e) {}
			}
			
			GameEngine.Renderer.canvas.oncontextmenu = function() {return false;};
		},
		
		clearScreen:function() {
			GameEngine.Renderer.context2D.clearRect(0, 0, GameEngine.Renderer.canvas.width, GameEngine.Renderer.canvas.height);
		},
		
		setBackgroundColor:function(color) {
			GameEngine.Renderer.canvas.style.backgroundColor = color;
		},
		
		setHardwareEnabled:function(enabled) {
			GameEngine.Renderer._hardwareAccelerated = enabled;
			// TODO: Enable hardware acceleration.
		},
		
		setCanvasSize:function(width, height) {
			GameEngine.Renderer.canvas.style.width = width;
			GameEngine.Renderer.canvas.style.height = height;
			
			GameEngine.Renderer.canvas.width = width;
			GameEngine.Renderer.canvas.height = height;
			
			if(GameEngine.Renderer.contextGL) {
				GameEngine.Renderer.contextGL.viewportWidth = width;
				GameEngine.Renderer.contextGL.viewportHeight = height;
			}
		},
		
		setFullscreen:function(fullscreen) {
			if(fullscreen) {
				try { GameEngine.Renderer.canvas.webkitRequestFullScreen(Element.ALLOW_KEYBOARD_INPUT); } catch(e) {}
				try { GameEngine.Renderer.canvas.mozRequestFullScreen(); } catch(e) {}
			} else {
				try { document.webkitCancelFullScreen(); } catch(e) {}
				try { document.mozCancelFullScreen(); } catch(e) {}
			}
		},
		
		setFramerate:function(fps) {
			GameEngine._framerate = fps;
			GameEngine._resetGameFunction();
		}
	},
	
	Media: {
		// Classes
		Audio:function(path) {
			// AUDIO FILE START
			var audioFile = new Audio();
			var alreadySignaled = false;
			GameEngine._totalObjects++;
			
			// Do not use audioFile.oncanplaythrough, there is a bug on Chromium on this event.
			audioFile.addEventListener("canplaythrough", function() {
				if(alreadySignaled)
					return;
				
				alreadySignled = true;
				GameEngine._loadedObjects++;
				GameEngine._resetGameFunction();
			}, false);
			audioFile.onerror = function() {
				throw { 
					name: "AudioLoadException", 
					message: "Failed to load \""+audioFile.src+"\" audio file!", 
				}
				
				GameEngine._loadedObjects++;
				GameEngine._resetGameFunction();
			};
			audioFile.src = path;
			audioFile.load();
			// AUDIO FILE END
			
			// Methods
			this.play = function() {
				audioFile.play();
			},
			
			this.pause = function() {
				audioFile.pause();
			},
			
			this.stop = function() {
				audioFile.pause();
				audioFile.currentTime = 0;
			}
		}
	},
	
	Keyboard: {
		// Static Classes
		KeyboardEvent: {
			// Attributes
			// EVENT_PRESS = 0
			// EVENT_RELEASE = 1
			// KEY_BACKSPACE = 8
			// KEY_TAB = 9
			// KEY_ENTER = 13
			// KEY_SHIFT = 16
			// KEY_CTRL = 17
			// KEY_ALT = 18
			// KEY_PAUSE = 19
			// KEY_ESCAPE = 27
			// KEY_SPACE = 32
			// KEY_PAGEUP = 33
			// KEY_PAGEDOWN = 34
			// KEY_END = 35
			// KEY_HOME = 36
			// KEY_LEFT = 37
			// KEY_UP = 38
			// KEY_RIGHT = 39
			// KEY_DOWN = 40
			// KEY_PRINT = 44
			// KEY_INSERT = 45
			// KEY_DELETE = 46
			// KEY_0 = 48
			// KEY_1 = 49
			// [...]
			// KEY_9 = 57
			// KEY_A = 65
			// KEY_B = 66
			// [...]
			// KEY_Z = 90
			// KEY_NUMPAD0 = 96
			// KEY_NUMPAD1 = 97
			// [...]
			// KEY_NUMPAD9 = 105
			// KEY_F1 = 112
			// KEY_F2 = 113
			// [...]
			// KEY_F12 = 123
			// KEY_NUMLOCK = 144
			// KEY_SCROLLLOCK = 145
		},
		
		// Classes
		KeyboardEvent:function(keyNumber, eventType) {
			var keyNum = keyNumber;
			var evType = eventType;
			
			// Methods
			this.getKeyNumber = function() {
				return keyNum;
			},
			
			this.getEventType = function() {
				return evType;
			}
		},
		
		// Attributes
		//_keyboardListeners
		
		// Methods
		addKeyboardListener:function(listener) {
			GameEngine.Keyboard._keyboardListeners.push(listener);
		},
		
		removeKeyboardListener:function(listener) {
			GameEngine._removeArrayElement(GameEngine.Keyboard._keyboardListeners, listener);
		},
		
		_initialize:function() {
			var kbe = GameEngine.Keyboard.KeyboardEvent;
			
			kbe.EVENT_PRESS = 0;
			kbe.EVENT_RELEASE = 1;
			
			kbe.KEY_BACKSPACE = 8;
			kbe.KEY_TAB = 9;
			kbe.KEY_ENTER = 13;
			kbe.KEY_SHIFT = 16;
			kbe.KEY_CTRL = 17;
			kbe.KEY_ALT = 18;
			kbe.KEY_PAUSE = 19;
			kbe.KEY_ESCAPE = 27;
			kbe.KEY_SPACE = 32;
			kbe.KEY_PAGEUP = 33;
			kbe.KEY_PAGEDOWN = 34;
			kbe.KEY_END = 35;
			kbe.KEY_HOME = 36;
			kbe.KEY_LEFT = 37;
			kbe.KEY_UP = 38;
			kbe.KEY_RIGHT = 39;
			kbe.KEY_DOWN = 40;
			kbe.KEY_PRINT = 44;
			kbe.KEY_INSERT = 45;
			kbe.KEY_DELETE = 46;
			for(var i=0; i<9; i++)
				kbe["KEY_"+i] = i+48;
			for(var i=0; i<26; i++)
				kbe["KEY_"+String.fromCharCode(65+i)] = i+65;
			for(var i=0; i<9; i++)
				kbe["KEY_NUMPAD"+i] = i+96;
			for(var i=0; i<12; i++)
				kbe["KEY_F"+i] = i+96;
			kbe.KEY_NUMLOCK = 144;
			kbe.KEY_SCROLLLOCK = 145;
			
			GameEngine.Keyboard._keyboardListeners = new Array();

			function upDateKeyDown(e) {
				GameEngine._sendEventToElements(GameEngine.Keyboard._keyboardListeners, new GameEngine.Keyboard.KeyboardEvent(e.keyCode, GameEngine.Keyboard.KeyboardEvent.EVENT_PRESS));
				
				// Prevent page scrolling or other key-related events.
				if(GameEngine.Keyboard._keyboardListeners.length > 0)
					e.preventDefault();
			}
			function upDateKeyUp(e) {
				GameEngine._sendEventToElements(GameEngine.Keyboard._keyboardListeners, new GameEngine.Keyboard.KeyboardEvent(e.keyCode, GameEngine.Keyboard.KeyboardEvent.EVENT_RELEASE));
			}
			
			document.addEventListener("keydown", upDateKeyDown, false);
			document.addEventListener("keyup", upDateKeyUp, false);
		}
	},
	
	Mouse: {
		// Static Classes
		MouseEvent: {
			// Attributes
			// EVENT_PRESS = 0
			// EVENT_RELEASE = 1
			// EVENT_MOVE = 2
			// EVENT_WHEEL = 3
			// MOUSE_LEFT = 0
			// MOUSE_RIGHT = 2
			// MOUSE_MIDDLE = 1
		},
		
		// Classes
		MouseEvent:function(buttonNumber, eventType) {
			var btnNum = buttonNumber;
			var evType = eventType;
			
			// Methods
			this.getButtonNumber = function() {
				return btnNum;
			},
			
			this.getWheelValue = function() {
				return btnNum;
			},
			
			this.getEventType = function() {
				return evType;
			}
		},
		
		// Attibutes
		//x
		//y
		//_mouseListeners
		//_wasDown
		//_mouseDown
		
		// Methods
		addMouseListener:function(listener) {
			GameEngine.Mouse._mouseListeners.push(listener);
		},
		
		removeMouseListener:function(listener) {
			GameEngine._removeArrayElement(GameEngine.Mouse._mouseListeners, listener);
		},
		
		_initialize:function() {
			GameEngine.Mouse.MouseEvent.EVENT_PRESS = 0;
			GameEngine.Mouse.MouseEvent.EVENT_RELEASE = 1;
			GameEngine.Mouse.MouseEvent.EVENT_MOVE = 2;
			GameEngine.Mouse.MouseEvent.EVENT_WHEEL = 3;
			GameEngine.Mouse.MouseEvent.MOUSE_LEFT = 0;
			GameEngine.Mouse.MouseEvent.MOUSE_RIGHT = 2;
			GameEngine.Mouse.MouseEvent.MOUSE_MIDDLE = 1;
			GameEngine.Mouse._mouseListeners = new Array();
			GameEngine.Mouse.x = 0;
			GameEngine.Mouse.y = 0;
			GameEngine.Mouse._wasDown = false;
			GameEngine.Mouse._mouseDown = false;
		},
		
		_gamestartInitializeOnce:function() {
			function upDateMouse(e) {
				var pos = GameEngine._getRelativePosition(e);
				GameEngine.Mouse.x = pos[0];
				GameEngine.Mouse.y = pos[1];
				
				GameEngine._sendEventToElements(GameEngine.Mouse._mouseListeners, new GameEngine.Mouse.MouseEvent(-1, GameEngine.Mouse.MouseEvent.EVENT_MOVE));
			}
			function upDateMouseDown(e) {
				if(e.button == 0)
					GameEngine.Mouse._mouseDown = true;
				
				e.preventDefault();
				
				GameEngine._sendEventToElements(GameEngine.Mouse._mouseListeners, new GameEngine.Mouse.MouseEvent(e.button, GameEngine.Mouse.MouseEvent.EVENT_PRESS));
			}
			function upDateMouseUp(e) {
				if(e.button == 0) {
					GameEngine.Mouse._mouseDown = false;
					GameEngine.Mouse._wasDown = true;
				}
				
				e.preventDefault();
				
				GameEngine._sendEventToElements(GameEngine.Mouse._mouseListeners, new GameEngine.Mouse.MouseEvent(e.button, GameEngine.Mouse.MouseEvent.EVENT_RELEASE));
			}
			function upDateWheel(e) {
				var evt = window.event || e;
				var delta = (evt.detail)?(evt.detail*(-120)):(evt.wheelDelta);
				
				GameEngine._sendEventToElements(GameEngine.Mouse._mouseListeners, new GameEngine.Mouse.MouseEvent(delta/120, GameEngine.Mouse.MouseEvent.EVENT_WHEEL));
				
				// Prevent page scrolling.
				if(GameEngine.Mouse._mouseListeners.length > 0)
					e.preventDefault();
				
				return false;
			}
			
			// SINPPET: http://www.javascriptkit.com/javatutors/onmousewheel.shtml
			var mousewheelevt = (/Firefox/i.test(navigator.userAgent))?("DOMMouseScroll"):("mousewheel");
			
			if(document.attachEvent)
				document.attachEvent("on"+mousewheelevt, upDateWheel);
			else if(document.addEventListener)
				document.addEventListener(mousewheelevt, upDateWheel, false);
			
			document.addEventListener("mousemove", upDateMouse, false);
			document.addEventListener("mouseup", upDateMouseUp, false);
			document.addEventListener("mousedown", upDateMouseDown, false);
		}
	},
	
	Touch: {
		// Static Classes
		TouchEvent: {
			// Attributes
			// EVENT_START = 0
			// EVENT_END = 1
			// EVENT_MOVE = 2
		},
		
		// Classes
		TouchEvent:function(touchID, eventType, x, y) {
			var tchID = touchID;
			var evType = eventType;
			
			// Attributes
			this.x = x;
			this.y = y;
			
			// Methods
			this.getTouchID = function() {
				return tchID;
			},
			
			this.getEventType = function() {
				return evType;
			}
		},
	
		// Attributes
		//_touchListeners
	
		// Methods
		addTouchListener:function(listener) {
			GameEngine.Touch._touchListeners.push(listener);
		},
		
		removeTouchListener:function(listener) {
			GameEngine._removeArrayElement(GameEngine.Touch._touchListeners, listener);
		},
		
		_initialize:function() {
			GameEngine.Touch.TouchEvent.EVENT_START = 0;
			GameEngine.Touch.TouchEvent.EVENT_END = 1;
			GameEngine.Touch.TouchEvent.EVENT_MOVE = 2;
			GameEngine.Touch._touchListeners = new Array();
		},
		
		_gamestartInitializeOnce:function() {
			function upDateTouch(e) {
				if(GameEngine.Touch._touchListeners.length > 0)
					e.preventDefault();
				
				for(var t in e.changedTouches) {
					p = e.changedTouches[t];
					
					if(p != "[object Touch]")
						continue;
					
					var pos = GameEngine._getRelativePosition(p);
					
					GameEngine._sendEventToElements(GameEngine.Touch._touchListeners, new GameEngine.Touch.TouchEvent(p.identifier, GameEngine.Touch.TouchEvent.EVENT_MOVE, pos[0], pos[1]));
				}
			}
			function upDateTouchStart(e) {
				if(GameEngine.Touch._touchListeners.length > 0)
					e.preventDefault();
				
				for(var t in e.changedTouches) {
					p = e.changedTouches[t];
					
					if(p != "[object Touch]")
						continue;

					var pos = GameEngine._getRelativePosition(p);
					
					GameEngine._sendEventToElements(GameEngine.Touch._touchListeners, new GameEngine.Touch.TouchEvent(p.identifier, GameEngine.Touch.TouchEvent.EVENT_START, pos[0], pos[1]));
				}
			}
			function upDateTouchEnd(e) {
				if(GameEngine.Touch._touchListeners.length > 0)
					e.preventDefault();
				
				for(var t in e.changedTouches) {
					p = e.changedTouches[t];
					
					if(p != "[object Touch]")
						continue;

					var pos = GameEngine._getRelativePosition(p);
					
					GameEngine._sendEventToElements(GameEngine.Touch._touchListeners, new GameEngine.Touch.TouchEvent(p.identifier, GameEngine.Touch.TouchEvent.EVENT_END, pos[0], pos[1]));
				}
			}
			
			document.addEventListener("touchmove", upDateTouch, false);
			document.addEventListener("touchstart", upDateTouchStart, false);
			document.addEventListener("touchend", upDateTouchEnd, false);
		}
	},
	
	Saving: {
		// Methods
		save:function(name, value) {
			localStorage[name] = value;
		},
		
		load:function(name) {
			return localStorage[name];
		}
	},
	
	GUI: {
		// Classes
		Button:function(x, y, width, height, text) {
			// Attributes
			this.x = x;
			this.y = y;
			this.width = width;
			this.height = height;
			this.text = text;
			this.enabled = true;
			
			// Methods
			this.render = function() {
				var c2d = GameEngine.Renderer.context2D;
				
				var mouseOver = GameEngine.GUI._isMouseOver(this.x, this.y, this.width, this.height);
				
				GameEngine.GUI._saveCanvasState();
				GameEngine.GUI._setCanvasStateToTheme(this.enabled, mouseOver);
				
				c2d.beginPath();
				c2d.moveTo(this.x+5, this.y);
				c2d.arcTo(this.x+this.width, this.y, this.x+this.width, this.y+this.height, 5);
				c2d.arcTo(this.x+this.width, this.y+this.height, this.x, this.y+this.height, 5);
				c2d.arcTo(this.x, this.y+this.height, this.x, this.y, 5);
				c2d.arcTo(this.x, this.y, this.x+this.width, this.y, 5);
				c2d.closePath();
				c2d.fill();
				
				GameEngine.GUI._setCanvasRemoveShadows();
				c2d.stroke();
				
				GameEngine.GUI._setCanvasPrepareText();
				c2d.textAlign = "center";
				c2d.textBaseline = "middle";
				c2d.fillText(this.text, this.x+this.width/2, this.y+this.height/2);
				
				GameEngine.GUI._loadCanvasState();
				
				if(GameEngine.Mouse._wasDown && mouseOver && this.enabled)
					return true;
				
				return false;
			}
		},
		
		CheckBox:function(x, y, text) {
			// Attributes
			this.x = x;
			this.y = y;
			this.text = text;
			this.enabled = true;
			this.selected = false;
			
			// Methods
			this.render = function() {
				var c2d = GameEngine.Renderer.context2D;
				
				var mouseOver = GameEngine.GUI._isMouseOver(this.x, this.y, 20, 20);
				
				GameEngine.GUI._saveCanvasState();
				GameEngine.GUI._setCanvasStateToTheme(this.enabled, mouseOver);
				
				c2d.beginPath();
				c2d.rect(this.x, this.y, 20, 20);
				c2d.closePath();
				c2d.fill();
				
				GameEngine.GUI._setCanvasRemoveShadows();
				c2d.stroke();
				
				GameEngine.GUI._setCanvasPrepareText();
				c2d.textAlign = "left";
				c2d.textBaseline = "middle";
				c2d.fillText(this.text, this.x+30, this.y+10);
				
				if(this.selected) {
					c2d.textAlign = "center";
					c2d.textBaseline = "middle";
					c2d.fillText("X", this.x+10, this.y+10);
				}
				
				GameEngine.GUI._loadCanvasState();
				
				if(GameEngine.Mouse._wasDown && mouseOver && this.enabled) {
					this.selected = !this.selected;
					return true;
				}
				
				return false;
			}
		},
		
		Label:function(x, y, width, height, text) {
			// Attributes
			this.x = x;
			this.y = y;
			this.width = width;
			this.height = height;
			this.text = text;
			this.align = "center";
			this.enabled = true;
			
			// Methods
			this.render = function() {
				var c2d = GameEngine.Renderer.context2D;
				
				var mouseOver = GameEngine.GUI._isMouseOver(this.x, this.y, 20, 20);
				
				GameEngine.GUI._saveCanvasState();
				GameEngine.GUI._setCanvasStateToTheme(this.enabled, mouseOver);
				GameEngine.GUI._setCanvasRemoveShadows();
				GameEngine.GUI._setCanvasPrepareText();
				c2d.textAlign = this.align;
				c2d.textBaseline = "middle";
				c2d.font = c2d.font.replace(/(\d+)/g, this.height);
				
				if(this.align == "center")
					c2d.fillText(this.text, this.x+this.width/2, this.y+this.height/2, this.width);
				else if(this.align == "right")
					c2d.fillText(this.text, this.x+this.width, this.y+this.height/2, this.width);
				else
					c2d.fillText(this.text, this.x, this.y+this.height/2, this.width);
				
				GameEngine.GUI._loadCanvasState();
				
				if(GameEngine.Mouse._wasDown && mouseOver && this.enabled)
					return true;
				
				return false;
			}
		},
		
		
		// Attributes
		//_csfs
		//_csss
		//_cssc
		//_cssb
		//_cssox
		//_cssoy
		//_cslc
		//_cslj
		//_cslw
		//_csml
		//_csf
		//_csta
		//_cstb
	
		// Methods
		_isMouseOver:function(x, y, w, h) {
			if(GameEngine.Mouse.x > x && GameEngine.Mouse.x < x+w && GameEngine.Mouse.y > y && GameEngine.Mouse.y < y+h)
				return true;
			return false;
		},
		
		_saveCanvasState:function() {
			var c2d = GameEngine.Renderer.context2D;
			
			GameEngine.GUI._csfs = c2d.fillStyle;
			GameEngine.GUI._csss = c2d.strokeStyle;
			GameEngine.GUI._cssc = c2d.shadowColor;
			GameEngine.GUI._cssb = c2d.shadowBlur;
			GameEngine.GUI._cssox = c2d.shadowOffsetX;
			GameEngine.GUI._cssoy = c2d.shadowOffsetY;
			GameEngine.GUI._cslc = c2d.lineCap;
			GameEngine.GUI._cslj = c2d.lineJoin;
			GameEngine.GUI._cslw = c2d.lineWidth;
			//GameEngine.GUI._csml = c2d.miterLimit;
			GameEngine.GUI._csf = c2d.font;
			GameEngine.GUI._csta = c2d.textAlign;
			GameEngine.GUI._cstb = c2d.textBaseline;
		},
		
		_setCanvasStateToTheme:function(enabled, mouseOver, mouseDown) {
			var c2d = GameEngine.Renderer.context2D;
			
			c2d.fillStyle = "#D9D9FF";
			c2d.strokeStyle = "#5959DF";
			c2d.shadowColor = "#292959";
			c2d.shadowBlur = 0;
			c2d.shadowOffsetX = 0;
			c2d.shadowOffsetY = 0;
			c2d.lineCap = "round";
			c2d.lineJoin = "rounded";
			c2d.lineWidth = 2;
			//c2d.miterLimit = 0;
			c2d.font = "20px Courier";

			if(!enabled) {
				c2d.fillStyle = "#D9D9D9";
				c2d.strokeStyle = "#595959";
			} else if(GameEngine.Mouse._mouseDown && mouseOver) {
				c2d.fillStyle = "#FFFFFF";
			} else if(mouseOver) {
				c2d.fillStyle = "#E9E9FF";
			}
		},
		
		_setCanvasRemoveShadows:function() {
			var c2d = GameEngine.Renderer.context2D;
			
			c2d.shadowColor = "";
			c2d.shadowBlur = 0;
			c2d.shadowOffsetX = 0;
			c2d.shadowOffsetY = 0;
		},
		
		_setCanvasPrepareText:function() {
			var c2d = GameEngine.Renderer.context2D;
			
			c2d.fillStyle = c2d.strokeStyle;
		},
		
		_loadCanvasState:function() {
			var c2d = GameEngine.Renderer.context2D;
			var gui = GameEngine.GUI;
			
			c2d.fillStyle = gui._csfs;
			c2d.strokeStyle = gui._csss;
			c2d.shadowColor = gui._cssc;
			c2d.shadowBlur = gui._cssb;
			c2d.shadowOffsetX = gui._cssox;
			c2d.shadowOffsetY = gui._cssoy;
			c2d.lineCap = gui._cslc;
			c2d.lineJoin = gui._cslj;
			c2d.lineWidth = gui._cslw;
			//c2d.miterLimit = gui._csml;
			c2d.font = gui._csf;
			c2d.textAlign = gui._csta;
			c2d.textBaseline = gui._cstb;
		}
	},
	
	// Attributes
	//_totalObjects
	//_loadedObjects
	//_gameFunction
	//_gameCycleIntervalID
	//_framerate
	//_firstTry
	
	// Methods
	startGame:function(gameFunction) {
		GameEngine._gameFunction = gameFunction;
		GameEngine._resetGameFunction();
	},
	
	_resetGameFunction:function() {
		if(GameEngine._gameCycleIntervalID) {
			clearInterval(GameEngine._gameCycleIntervalID);
			GameEngine._gameCycleIntervalID = undefined;
		}
		
		if(GameEngine._loadedObjects >= GameEngine._totalObjects && GameEngine._gameFunction) {
			if(GameEngine._firstTry) {
				GameEngine.Mouse._gamestartInitializeOnce();
				GameEngine.Touch._gamestartInitializeOnce();
				GameEngine._firstTry = false;
			}
			
			GameEngine._gameCycleIntervalID = setInterval(function() {
				//try {
					GameEngine._gameFunction();
				//} catch(error) {
					// If there is an exception here, stop the engine...
					//clearInterval(GameEngine._gameCycleIntervalID);
					//GameEngine._gameCycleIntervalID = undefined;
				//}
				
				GameEngine.Mouse._wasDown = false;
			}, 1000/GameEngine._framerate);
		}
	},
	
	_initialize:function() {
		GameEngine._totalObjects = 0;
		GameEngine._loadedObjects = 0;
		GameEngine._framerate = 20;
		GameEngine._firstTry = true;
		
		GameEngine.Keyboard._initialize();
		GameEngine.Mouse._initialize();
		GameEngine.Touch._initialize();
	},
	
	_removeArrayElement:function(arrayName, arrayElement) {
		for(var i=0; i<arrayName.length; i++) { 
			if(arrayName[i]==arrayElement)
				arrayName.splice(i,1);
		} 
	},
	
	_sendEventToElements:function(array, event) {
		for(var i = 0; i < array.length; i++) {
			array[i](event);
		}
	},
	
	_getRelativePosition:function(e) {
		// SNIPPET: http://tinyurl.com/3r6kcqf
		var x, y;
		if (e.pageX || e.pageY) {
			x = e.pageX;
			y = e.pageY;
		} else {
			x = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
			y = e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
		}

		x -= GameEngine.Renderer.canvas.offsetLeft;
		y -= GameEngine.Renderer.canvas.offsetTop;

		return [x,y];
	}
}
GameEngine._initialize();